/**
 *
 * Copyright (c) 2007 Zurschmiede Lukas
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 *
 *
 * ComicMaker:ComicMaker_ComicImage
 * http://www.delight.ch/
 * Zurschmiede Lukas
 * delight software gmbh
 * Version 1.0 (05-07-2007)
 *
 **/
function ComicMaker_ImageChooser(id, opt, optname) {
	this.contentId = id;
	this.opt = opt;
	this.optName = optname;
	this.width = this.opt.get(this.optName, 400, 'width');
	this.height = this.opt.get(this.optName, 175, 'height');
	this.images = {};
	this.thumbnails = {};
	this.imagesLoaded = 0;
	this.imagesTotal = 0;
	this.init();
}

ComicMaker_ImageChooser.prototype = {
	init : function() {
		if ($(this.contentId)) {
			var title = this.opt.get(this.optName, 'Choose one', 'title');
			$(this.contentId).innerHTML = '<div class="border" style="height:'+this.height+'px;width:'+this.width+'px;" id="'+this.contentId+'_par">' +
				'<h1 class="content" id="'+this.contentId+'_title">' + title + '</h1>' +
				'<div class="autoover nobr middle left" id="'+this.contentId+'_cont"></div>'
				'</div>';
		}
	},

	imageLoaded : function(img) {
		this.imagesLoaded++;
		if (this.imagesLoaded == this.imagesTotal) {
			this.showLoadedThumbnails();
			this.observeLoadedThumbnails();
		}
	},

	loadThumbnails : function() {
		var path = this.opt.get(this.optName, false, 'images');
		var figures = this.opt.get(this.optName, false, 'figures');
		var fext = this.opt.get(this.optName, 'png', 'extension');
		var obj = this;
		if (figures) {
			figures.each(function(val, k) {
				obj.thumbnails[val[0]] = new ComicMaker_ComicImage(val[0], path + val[0] + '.' + fext, obj.contentId);
				obj.imagesTotal++;
			});
		}
	},

	loadImages : function() {
		var path = this.opt.get(this.optName, false, 'images');
		var figures = this.opt.get(this.optName, false, 'figures');
		var fext = '.' + this.opt.get(this.optName, 'png', 'extension');
		var begin = this.opt.get(this.optName, 0, 'begin');
		var i,img,fig,num;
		var obj = this;
		if (figures) {
			figures.each(function(val, k) {
				fig = val[0];
				num = val[1];
				obj.images[fig] = {};
				for (i = 0; i < num; i++) {
					obj.images[fig][i] = new ComicMaker_ComicImage(fig, path + fig + '/' + fig + (i+begin) + fext, obj.contentId);
					obj.imagesTotal++;
				}
			});
		}
	},

	showLoadedThumbnails : function() {
		var obj = this;
		var img = '';
		$H(this.thumbnails).each(function(thumb) {
			if (!thumb.value.imageError()) {
				img = '<img src="' + thumb.value.getImageSource() + '" class="thumbnail" id="' + thumb.value.getImageId() + '" alt="' + thumb.key + '" longdesc="' + obj.contentId + '" />';
				$(obj.contentId + '_cont').innerHTML += img;
			}
		});
	},

	observeLoadedThumbnails : function() {
		$H(this.thumbnails).each(function(thumb) {
			if ($(thumb.value.getImageId())) {
				Event.observe($(thumb.value.getImageId()), 'mouseup', thumbnailClick.preview.bindAsEventListener(this));
			}
		});
	},

	getImage : function(num, base) {
		if (this.images[base]) {
			var imgNum = $H(this.images[base]).keys().length;
			while (num >= imgNum) {
				num -= imgNum;
			}
			while (num < 0) {
				num += imgNum;
			}
			if (this.images[base][num]) {
				return this.images[base][num];
			}
		}
		return null;
	},

	isBackground : function() {
		return this.opt.get(this.optName, false, 'background');
	},

	isEditable : function() {
		return this.opt.get(this.optName, false, 'textedit');
	},

	isResizable : function() {
		return this.opt.get(this.optName, true, 'resizable');
	},

	last : null
};