/**
*
* Copyright (c) 2007 Zurschmiede Lukas
*
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without
* restriction, including without limitation the rights to use, copy,
* modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
* BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
* ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
*
*
* ComicMaker:ComicMaker_ComicStage
* http://www.delight.ch/
* Zurschmiede Lukas
* delight software gmbh
* Version 1.0 (05-07-2007)
*
**/
function ComicMaker_ComicStage(width, height) {
	this.opt = new djs_Properties();
	this.opt.set('width', width ? parseInt(width) : $('stage') ? $('stage').getWidth() : 1024);
	this.opt.set('height', height ? parseInt(height) : $('stage') ? $('stage').getHeight() : 768);
	this.opt.set('stageid', 'stage');
	this.stageElements = {};
	this.stages = {};
	this.draggables = {};
	this.cookieMaxSize = 4095; // normally 4096 but this makes some problems with FireFox and probably also with others
	this.loadedStageObjects = {};
	this.printWindow = [];
}

/**
* All functions for this Object
*/
ComicMaker_ComicStage.prototype = {
	/**
	* Initialize the Object
	* param Object prop Properties to use
	*/
	init : function(prop) {
		this.opt.parse(prop);
		if (this.getStage() == null) {
			this.createStage();
		}
		this.loadStageImages();
	},
	/** afterEvent
	* This function is called after the whole Content is loaded
	*/
	show : function() {
		this.resizeAndShowContent();
	},
	/** NOT USED CURRENTLY
	* Set the width for XY
	* param integer width Width to set
	*/
	setWidth : function(width) {
		this.opt.set('fieldwidth', parseInt(width))
	},
	/** NOT USED CURRENTLY
	* Set the height for YX
	* param integer height Height to set
	*/
	setHeight : function(height) {
		this.opt.set('fieldheight', parseInt(height))
	},
	/**
	* Check for the Body-Element end return it
	* return HTMLBodyElement The BodyElement
	*/
	getBody : function() {
		try {
			return document.getElementsByTagName('body')[0];
		} catch(e) {
			alert('We cannot create the stage because we cannot find the Body-Element.\nCheck the HTML for a <body>-Tag.');
		}
	},
	/**
	* Return the main ComicMaker stage (not the Stage where we can drop images, here the stage is the stage where we draw our ComicMaker on)
	* return HTMLElement If found, the ComicMaker-Stage, null otherwise
	*/
	getStage : function() {
		if ($(this.opt.get('stageid'))) {
			return $(this.opt.get('stageid'));
		} else {
			return null;
		}
	},
	/**
	* Create and load all needed components for the ComicMaker and show them
	*/
	createStage : function() {
		this.createStageContent();
		this.showMainStoryElements();
		this.stageElements.figures = new ComicMaker_ImageChooser('figures', this.opt, 'FigureContainer');
		this.stageElements.prompts = new ComicMaker_ImageChooser('prompts', this.opt, 'PromptsContainer');
		this.stageElements.balloons = new ComicMaker_ImageChooser('balloons', this.opt, 'BalloonContainer');
		this.stageElements.backgrounds = new ComicMaker_ImageChooser('backgrounds', this.opt, 'BackgroundContainer');
		this.stageElements.preview = new ComicMaker_ImagePreview('preview', this.opt, 'FigurePreview');
		this.addNewStage();
	},
	/**
	* Load all Images inside all ComicChoosers
	*/
	loadStageImages : function() {
		this.stageElements.figures.loadThumbnails();
		this.stageElements.figures.loadImages();
		this.stageElements.prompts.loadThumbnails();
		this.stageElements.prompts.loadImages();
		this.stageElements.balloons.loadThumbnails();
		this.stageElements.balloons.loadImages();
		this.stageElements.backgrounds.loadThumbnails();
		this.stageElements.backgrounds.loadImages();
	},
	/**
	* Create the main Content for ComicMaker and update the Body after...
	*/
	createStageContent : function() {
		var body = this.getBody();
		body.setAttribute('id', this.opt.get('stageid', 'stage'));
		var html = '<div id="mainelements"><table cellpadding="0" cellspacing="5"><tr>' +
		'<td rowspan="2" id="preview"></td>' +
		'<td id="figures" valign="top"></td><td id="balloons" valign="top"></td>' +
		'</tr><tr>' +
		'<td id="backgrounds" valign="top"></td><td id="prompts" valign="top"></td>' +
		'</tr></table>' +
		'<div id="storryelements"></div></div>' +
		'<div id="stageelements"></div>';
		body.update(html);
	},
	/**
	* Create and show the main StoryElements - Storyname, Directors name, save-, load- and printbutton
	*/
	showMainStoryElements : function() {
		var html = '<table cellpadding="0" cellspacing="0"><tr>' +
		'<td style="width:360px;font-weight:bold;"><img src="images/layout/comicauthor.png" align="center" /> Your Name: <span id="directorname" onclick="ComicStage.makeEditable(this);"><em>Click here to enter your Name</em></span></td>' +
		'<td style="width:430px;font-weight:bold;"><img src="images/layout/comictitle.png" align="center" /> Name of this Story: <span id="storyname" onclick="ComicStage.makeEditable(this);"><em>Click here to enter the Storytitle</em></span></td>' +
		'<td style="width:260px;">' +
		'<div class="stagebutton" onclick="ComicStage.showSaveComic();"><img src="images/layout/save.png" /> Save</div>' +
		'<div class="stagebutton" onclick="ComicStage.showLoadComic();"><img src="images/layout/open.png" /> Open/Load</div>' +
		'<div class="stagebutton" onclick="ComicStage.printComic();"><img src="images/layout/print.png" /> Print</div>' +
		'</td>' +
		'</tr></table><br />';

		$('storryelements').update(html);
	},
	/**
	* Called after complete content is loaded, so we can make the whole ImageChooser-HTML floatable, etc.
	*/
	resizeAndShowContent : function() {
		if ($('mainelements')) {
			var main = $('mainelements');
			main.style.position = 'fixed';
			main.style.display = 'block';
			main.style.top = '0px';
			main.style.zIndex = 1;
			main.style.backgroundColor = 'white';
			this.getBody().style.paddingTop = (main.getHeight() + 10) + 'px';
		}
	},

	/** afterEvent
	* Called after an image is loaded inside an ComicMaker_ImageChooser - used for check if an Image could be loaded or not
	* param string chooser Name of the Chooser where all images are loaded sucessfully
	* param string image NAme of the Image which was loaded
	*/
	chooserImageLoaded : function(chooser, image) {
		if (this.stageElements[chooser] && (typeof(this.stageElements[chooser].imageLoaded) == 'function')) {
			this.stageElements[chooser].imageLoaded(image);
		}
	},

	/** onEvent
	* Called from an ImageChooser after clicking on a Image which then will be showed in ComicPreview
	* param string chooser Name of the ComicMaker_ImageChooser on which an Previewimage was clicked
	* param string img Name of the Image (longdesc-attribute) on which was clicked
	* param integer num Number of image which should be showed
	*/
	showPreviewImage : function(chooser, img, num) {
		if (this.stageElements[chooser] && (typeof(this.stageElements[chooser].getImage) == 'function')) {
			if (isNaN(num)) { num = 0; }
			this.stageElements.preview.showPreview(this.stageElements[chooser].getImage(num, img), num, chooser);
		}
	},
	/** onEvent
	* Calls ComicPreview to show us the next image
	*/
	showNextPreviewImage : function() {
		var num = this.stageElements.preview.getCurrentImageNumber() + 1;
		var chooser = this.stageElements.preview.getCurrentImageType();
		var img = this.stageElements.preview.getCurrentImageCat();
		this.showPreviewImage(chooser, img, num);
	},
	/** onEvent
	* Calls ComicPreview to show us the previous image
	*/
	showPreviousPreviewImage : function() {
		var num = this.stageElements.preview.getCurrentImageNumber() - 1;
		var chooser = this.stageElements.preview.getCurrentImageType();
		var img = this.stageElements.preview.getCurrentImageCat();
		this.showPreviewImage(chooser, img, num);
	},
	/**
	* Make a HTMLElement editable
	* param HTMLElement obj The HTMLElement to make Editable
	*/
	makeEditable : function(obj) {
		var cont = obj.innerHTML.stripTags();
		var date = new Date();
		date = date.getTime();
		obj.update('<input type="text" name="edt'+date+'" id="edt'+date+'" value="'+cont+'" onblur="ComicStage.disableEditable(this.parentNode, this.value)" />');
		$('edt' + date).focus();
		$('edt' + date).select();
	},
	/**
	* Removes the Editable-Flag from a HTMLElement which was made editable
	* param HTMLElement obj The main ContentElement where the Text should be set in
	* param string value the Value the user has entered
	*/
	disableEditable : function(obj, value) {
		if (value.length <= 0) {
			value = '<em>Click here and enter a value</em>';
		}
		obj.update(value);
	},

	/**
	* Show a Popup where the save-state will be visualized
	*/
	showSaveComic : function() {
		var cookieStages = [];
		var cookieElements = [];
		var cookieMain = [];
		var stagesCount = 0;
		var elementsCount = 0;
		$H(this.stages).each(function(args) {
			var stage = args.value;
			var stageId = args.key;
			var stagePos = stage.getPosition();
			var objects = stage.getStageElements();
			cookieStages.push([stageId,stagePos]);
			cookieElements.push([stageId,objects]);

			stagesCount++;
			elementsCount += objects['stage_1'].length;
			elementsCount += objects['stage_2'].length;
			elementsCount += objects['stage_3'].length;
		});

		// directorname, storyname
		var title = $('storyname').innerHTML;
		var name  = $('directorname').innerHTML;
		if (title.indexOf('Click here to enter') >= 0) {
			title = '';
		}
		if (name.indexOf('Click here to enter') >= 0) {
			name = '';
		}
		cookieMain.push(name);
		cookieMain.push(title);

		if (title == '' || name == '') {
			alert("Please enter first your name and a Storry-Title.");
		} else {
			if (window.location.protocol.indexOf('file') >= 0) {
				this.deleteCookie('comicMakerStages');
				this.deleteCookie('comicMakerElements');
				this.deleteCookie('comicMakerMain');
				this.setCookie('comicMakerStages', cookieStages.toJSON(), 31536000000);
				this.setCookie('comicMakerElements', cookieElements.toJSON(), 31536000000);
				this.setCookie('comicMakerMain', cookieMain.toJSON(), 31536000000);
				alert("saved " + stagesCount + " stages and " + elementsCount + " objects");
			} else {
				alert("Currently we can only save comics in offline installations.");
			}
		}
	},
	/**
	* Show a Popup where the User can choose between all saved storries
	*/
	showLoadComic : function() {
		if (window.location.protocol.indexOf('file') >= 0) {
			var stages = this.getCookie('comicMakerStages');
			var elements = this.getCookie('comicMakerElements');
			var mainelem = this.getCookie('comicMakerMain');
		} else {
			alert("Currently we can only load comics in offline installations");
		}
		var obj = this;
		if (stages.isJSON()) {
			// Set name and storytitle
			if (mainelem.isJSON()) {
				mainelem = mainelem.evalJSON();
				$('directorname').innerHTML = mainelem[0];
				$('storyname').innerHTML = mainelem[1];
				mainelem = [];
			}

			// remove all stages
			$H(this.stages).each(function(param) {
				param.value.remove();
			});
			this.stages = {};

			// create loaded stages and elements (JSON)
			stages = stages.evalJSON();
			this.loadedStageObjects = {};
			if (elements.isJSON()) {
				elements = elements.evalJSON();
				elements.each(function(elem) {
					obj.loadedStageObjects[elem[0]] = elem[1];
				});
			} else {
				elements = null;
			}
			// Sort stages by it's position
			stages.sortBy(function(val) {
				return parseInt(val[1]);
			});
			// load all Stages
			var obj = this;
			stages.each(function(val) {
				obj.addNewStage(val[0]);
			});
			this.doLoadStageContentAfterLoading();
		}
	},
	doLoadStageContentAfterLoading : function(doload) {
		if (doload) {
			this.updateStageContent();
		} else {
			var loading = 0,total = 0;
			var obj = this;
			$H(this.stages).each(function(param) {
				if (param.value.stageLoaded && param.value.stageFramesLoaded()) {
					param.value.stageLoaded = false;
					loading++;
					if (obj.loadedStageObjects && obj.loadedStageObjects[param.key]) {
						param.value.savedObjects = obj.loadedStageObjects[param.key];
					}
				}
				total++;
			});
			if (loading == total) {
				this.loadedStageObjects = {};
				window.setTimeout('ComicStage.doLoadStageContentAfterLoading(true)', 10);
			} else {
				window.setTimeout('ComicStage.doLoadStageContentAfterLoading()', 10);
			}
		}
	},
	/**
	* Open a new Window with the whole comic inside which is ready for printing
	*/
	printComic : function() {
		this.printWindow = [];
		for (var i = 0; i < Math.ceil($H(this.stages).keys().length / 5); i++) {
			this.printWindow.push(window.open('print.svg', 'PrintComic' + i, 'dependent=yes,hotkeys=no,location=no,menubar=yes,resizable=yes,scrollbars=yes,status=yes,toolbar=no,top=0,left=0'));
		}
		for (var i = 0; i < this.printWindow.length; i++) {
			this.doLoadPrintComic(i);
		}
	},
	doLoadPrintComic : function(num) {
		if (this.printWindow[num] && !this.printWindow[num].frameLoaded) {
			window.setTimeout('ComicStage.doLoadPrintComic('+num+')', 10);
		} else if (this.printWindow[num] && this.printWindow[num].frameLoaded) {
			var win = this.printWindow[num];

			// directorname, storyname
			if (win) {
				var title = $('storyname').innerHTML;
				var name  = $('directorname').innerHTML;
				win.setTitle((title.indexOf('Click here to enter') >= 0) ? '' : title);
				win.setName((name.indexOf('Click here to enter') >= 0) ? '' : name);
			}

			// load all Stages
			var obj = this;
			$H(this.stages).each(function(param) {
				if (win) {
					//var stage = {};
					//stage.stageId = param.value.stageId;
					//stage.stageNumber = param.value.stageNum;
					//stage.stagePosition = param.value.stagePosition;
					//stage.elements = param.value.getStageElements();
					//win.addNewStage(stage);
					var elements = param.value.getStageElements();
					var sid = win.addStage();
					['stage_1','stage_2','stage_3'].each(function(idx, key) {
						var obj, i;
						if ( (typeof(elements[idx]) != 'undefined') && (elements[idx].length > 0) ) {
							for (i = 0; i < elements[idx].length; i++) {
								obj = elements[idx][i];
								if (obj.background) {
									win.addStageBackground(sid, key, obj.image);
								} else {
									win.addStageImage(sid, key, obj.image, obj.top, obj.left, obj.width, obj.height, obj.imageText);
								}
							}
						}
					});
					//win.addStageBackground(sid, );
				}
			});
			if (win) {
				//win.showPrintStages();
				win.showComic(num+1);
			}
		}
	},

	/** beforeEvent
	* Makes an Element Draggable
	* param string id Makes Element with this id draggable
	*/
	registerDraggable : function(id) {
		if (this.draggables[id]) {
			this.unregisterDraggable(id);
		}
		var drag = new Draggable(id, {revert:true,ghosting:true});
		this.draggables[id] = drag;
	},
	/** afterEvent
	* Remove the Draggable-Event from an Element
	* Called for example on mouseout in ComicPreview
	* param string id ID from Element to remove the DraggableEvent
	*/
	unregisterDraggable : function(id) {
		if (this.draggables[id]) {
			this.draggables[id].destroy();
			this.draggables[id] = null;
		}
	},

	/** onEvent
	* Add a new Stage on bottom - in future releases we would add the stage on bottom of the clicked stage
	*/
	addNewStage : function(sid) {
		if (typeof(sid) == 'undefined') {
			var date = new Date();
			var stageId = 'stage' + date.getTime();
		} else {
			var stageId = sid;
		}
		this.stages[stageId] = new ComicMaker_ComicStageArea(stageId, 'stageelements', $H(this.stages).keys().length);
		this.stages[stageId].create();
		this.stages[stageId].setPosition($H(this.stages).keys().length);
	},
	/** onEvent
	* Called after dropping an image on a Stage - calls a function on the stage
	* param HTMLImage img Image-Object which was dropped on a stage
	* param string stageSection ID from StageSection (the real stage where an image could be dropped)
	*/
	appendToStageComponent : function(img, stageSection) {
		var stageId  = stageSection.id.substring(0, stageSection.id.indexOf('_'));
		var stageNum = stageSection.id.substring(stageSection.id.indexOf('_')+1);
		if (this.stages[stageId]) {
			this.unregisterDraggable(img.id);
			this.stages[stageId].appendImage(img, stageNum);
		}
	},
	/** onEvent
	* Remove the stage identified by stageId
	* param string stageId The Stage to remove
	*/
	deleteStage : function(stageId) {
		if (this.stages[stageId]) {
			var stagePos = this.stages[stageId].getPosition();
			this.stages[stageId].remove();
			var newStages = {};
			$H(this.stages).each(function (stage) {
				if (stage.key != stageId) {
					if (stage.value.getPosition() > stagePos) {
						stage.value.setPosition(stage.value.getPosition() - 1);
					}
					newStages[stage.key] = stage.value;
				}
			});
			this.stages = newStages;
			if ($H(this.stages).keys().length <= 0) {
				this.addNewStage();
			}
		}
	},
	/**
	* Move a Stage up or down
	* param string stageId ID of Stage to moove up or down
	* param string action Can be "up" or "down" - Direction to move the stage
	*/
	moveStage : function(stageId, action) {
		if (this.stages[stageId]) {
			action = action.toLowerCase();
			if (action == 'up') {
				var changeWith = $(stageId).previous('div');
			} else {
				var changeWith = $(stageId).next('div');
			}
			if ( (typeof(changeWith) != 'undefined') && (changeWith != $(stageId)) ) {
				var changeStageId = changeWith.getAttribute('id');

				if (action == 'up') {
					this.stages[stageId].setPosition(this.stages[stageId].getPosition() - 1);
					this.stages[changeStageId].setPosition(this.stages[changeStageId].getPosition() + 1);
					this.stages[stageId].changePosition(this.stages[changeStageId]);
				} else {
					this.stages[stageId].setPosition(this.stages[stageId].getPosition() + 1);
					this.stages[changeStageId].setPosition(this.stages[changeStageId].getPosition() - 1);
					this.stages[changeStageId].changePosition(this.stages[stageId]);
				}
			}
		}
	},
	/**
	* Call UpdateFunction on one or all Stages if they are loaded
	* param string stageId optional StageId to load
	*/
	updateStageContent : function(stageId) {
		if (typeof(stageId) == 'undefined') {
			var loaded = 0;
			$H(this.stages).each(function(param) {
				if (!param.value.stageLoaded && param.value.stageFramesLoaded()) {
					param.value.restore();
					loaded++;
				}
			});
			if (loaded < $H(this.stages).keys().length) {
				window.setTimeout('ComicStage.updateStageContent()', 1);
			}
		} else {
			if (!this.stages[stageId].stageLoaded && this.stages[stageId].stageFramesLoaded()) {
				this.stages[stageId].restore();
			} else {
				window.setTimeout('ComicStage.updateStageContent(\''+stageId+'\')', 1);
			}
		}
	},

	/**
	* Set a Cookie
	* param string name Name for the Cookie
	* param string value Value for the Cookie
	* param int expires Milliseconds after which this cookie should expire
	*/
	setCookie : function(name, value, expires) {
		var cexpires = new Date();
		cexpires = new Date(cexpires.getTime() + expires);
		value = escape(value);

		var cookieOffsetSize = name + '= ; expires = ' + cexpires.toGMTString() + ' ; secure = yes; host = ; Path = ;';
		cookieOffsetSize = cookieOffsetSize.length;

		if (value.length > this.cookieMaxSize - cookieOffsetSize) {
			// we can have max. 300 cookies, so the name will be max 3 chars longer
			// the cookie holds also the Pathname and the Domainname, so we must also shorten the length by the length of them
			var maxCookieValueLength = this.cookieMaxSize - cookieOffsetSize - 3 - window.location.href.length;
			var cval = value.substr(0, maxCookieValueLength);
			var coff = 0;
			while (cval.length > 0) {
				document.cookie = name + coff + '=' + cval + ';expires=' + cexpires.toGMTString() + ';';
				coff++;
				cval = value.substr((coff * maxCookieValueLength), maxCookieValueLength);
			}
		} else {
			document.cookie = name + '=' + value + ';expires=' + cexpires.toGMTString() + ';';
		}
	},
	/**
	* Get a Cookievalue
	* param string name Name of the Cookie to get
	* return string Vlaue from Cookie identified by name
	*/
	getCookie : function(name) {
		var cvalue = null;
		if (document.cookie) {
			if (document.cookie.indexOf(name + '=') > -1) {
				var cpos = document.cookie.indexOf(name + '=');
				var cend = document.cookie.indexOf(';', cpos);
				if (cend < cpos) {
					cend = document.cookie.length;
				}
				cvalue = document.cookie.substring(cpos + name.length + 1, cend);
			} else {
				var coff = 0;
				var cvalue = '', cname = name + coff;
				var cend, cpos = document.cookie.indexOf(cname + '=');
				while (cpos > -1) {
					cend = document.cookie.indexOf(';', cpos);
					if (cend < cpos) {
						cend = document.cookie.length;
					}
					cvalue += document.cookie.substring(cpos + cname.length + 1, cend);
					coff++;
					cname = name + coff;
					cpos = document.cookie.indexOf(cname + '=');
				}
			}
			cvalue = unescape(cvalue);
		}
		return cvalue;
	},
	/**
	* Delete a Cookie
	* param string name Delete the Cookie with this Name
	*/
	deleteCookie : function(name) {
		if (document.cookie) {
			if (document.cookie.indexOf(name + '=') > -1) {
				this.setCookie(name, '', -31536000000);
			} else {
				var coff = 0;
				var cname = name + coff;
				while (document.cookie.indexOf(cname + '=') > -1) {
					this.setCookie(cname, '', -31536000000);
					coff++;
					cname = name + coff;
				}
			}
		}
	},

	last : null
};

var ComicStage = new ComicMaker_ComicStage();