/**
 *
 * Copyright (c) 2007 Zurschmiede Lukas
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 *
 *
 * ComicMaker:ComicMaker_ComicStageArea
 * http://www.delight.ch/
 * Zurschmiede Lukas
 * delight software gmbh
 * Version 1.0 (05-07-2007)
 *
 **/
function ComicMaker_ComicStageArea(id, parent, number) {
	this.stageId = id;
	this.stageParent = parent;
	this.stageNum = number ? number : 0;
	this.savedObjects = {};
	this.stagePosition = 0;
	this.stageLoaded = false;
	this.isPrintContent = false;
}

/**
 * All functions for this Object
 */
ComicMaker_ComicStageArea.prototype = {
	/**
	 * Create a Stage and append it to this.stageParent
	 * param boolean returnCont Do not create Iframes - needed while moveing stages
	 */
	create : function(returnCont) {
		if ($(this.stageParent)) {
			this.stageLoaded = true;
			var html = '<table cellbadding="0" cellspacing="0" id="'+this.stageId+'_cont" style="margin-left:-10px;"><tr>' +
			           '<td id="'+this.stageId+'_f">' +
			           '<img src="images/layout/stageup.png" id="'+this.stageId+'_sup" onclick="ComicStage.moveStage(\''+this.stageId+'\',\'up\');" />' +
			           '<br/><br/><br/><br/><br/><br/><br/><br/><br/>' +
			           '<img src="images/layout/stagedown.png" id="'+this.stageId+'_sdown" onclick="ComicStage.moveStage(\''+this.stageId+'\',\'down\');" />' +
			           '</td>' +
			           '<td id="'+this.stageId+'_1" class="droparea1"></td>' +
			           '<td id="'+this.stageId+'_2" class="droparea2"></td>' +
			           '<td id="'+this.stageId+'_3" class="droparea3"></td>' +
			           '</tr><tr><td>&nbsp;</td><td colspan="3">' +
			           '<div class="stagebutton" onclick="ComicStage.addNewStage();"><img src="images/layout/stageadd.png" style="width:16px;height:16px;" /> Add a new Stage</div>' +
			           '<div class="stagebutton" onclick="ComicStage.deleteStage(\''+this.stageId+'\');"><img src="images/layout/stagedelete.png" style="width:16px;height:16px;" /> Delete upper Stage</div>' +
			           '</td></tr></table>';
			var div = document.createElement('div');
			div.id = this.stageId;
			div.innerHTML = html;

			// Return the div if we move the stages
			if ((typeof(returnCont) == 'undefined') || !returnCont) {
				$(this.stageParent).appendChild(div);
				$(this.stageId).addClassName('stagearea');

				this.createIframe(null,null, this.stageId+'_c1', this.stageId+'_1');
				this.createIframe(null,null, this.stageId+'_c2', this.stageId+'_2');
				this.createIframe(null,null, this.stageId+'_c3', this.stageId+'_3');

				this.makeDropable();
			} else {
				return div;
			}
		}
	},
	/**
	 * Restore all Objects in all stages, based on the informations we have in this.savedObjects
	 */
	restore : function() {
		var obj = this;
		['stage_1','stage_2','stage_3'].each(function(idx, key) {
			var objects;
			var stage = obj.getIframeWin(obj.stageId + '_c' + (key + 1));
			if (stage && obj.savedObjects[idx]) {
				for (var i = 0; i < obj.savedObjects[idx].length; i++) {
					object = obj.savedObjects[idx][i];
					// {background:false, image:'', name:this.name, id:0, top:0, left:0, width:0, height:0, pos:0, stage:this.stageId, imageText:'', imageTextEditable:false }
					if (object.background) {
						stage.setBackground(object.image, obj.isPrintContent);
					} else {
						stage.restoreImage(object.image, object.name, object.stage, object.id, object.pos, object.left, object.top, object.width, object.height, object.imageText, obj.isPrintContent);
					}
				}
			}
		});
	},
	/**
	 * Remove this stage - We remove all HTML-Elements
	 */
	remove : function() {
		this.removeDropables();
		// we remove the Iframes standalone just to be sure they are destroyed
		$(this.stageId + '_c1').remove();
		$(this.stageId + '_c2').remove();
		$(this.stageId + '_c3').remove();
		$(this.stageId).remove();
	},
	/**
	 * Just set the Position
	 * param integer pos StagePosition on screen
	 */
	setPosition : function(pos) {
		this.stagePosition = pos;
	},
	/**
	 * Just return the Position
	 * return integer StagePosition on screen
	 */
	getPosition : function(pos) {
		return this.stagePosition;
	},
	/**
	 * Change the Position with the given other stage
	 * param ComicMaker_ComicStageArea changeWith Change the Position with this StageArea
	 */
	changePosition : function(changeWith) {
		// Get all Stage-Settings (Images and it's places)
		changeWith.savedObjects = changeWith.getStageElements();

		// Remove the old HTML-StageArea
		changeWith.remove();

		// First we create the new Stage and then we moove the IFrames to this stage - after we can delete the old StageContent
		newCont = changeWith.create(true);
		newCont = $(this.stageId).insertAfter(newCont);

		// Create all IFrames on the new Stage
		changeWith.createIframe(null,null, changeWith.stageId+'_c1', changeWith.stageId+'_1');
		changeWith.createIframe(null,null, changeWith.stageId+'_c2', changeWith.stageId+'_2');
		changeWith.createIframe(null,null, changeWith.stageId+'_c3', changeWith.stageId+'_3');

		// Make the Stages Dropable and set a new Class for the StageArea
		changeWith.makeDropable();
		$(changeWith.stageId).addClassName('stagearea');
		changeWith.stageLoaded = false;
		window.setTimeout('ComicStage.updateStageContent(\''+changeWith.stageId+'\')', 10);
	},
	/**
	 * Show this stage as a printable stage - no drag'n'drop, no buttons, etc.
	 * param string container Name of the main Stage-Container
	 */
	showPrintableStage : function() {
		var cont = $(this.stageParent);
		this.stageLoaded = false;
		if (cont) {
			var html = '<table cellbadding="0" cellspacing="0" id="'+this.stageId+'_cont"><tr>' +
			           '<td id="'+this.stageId+'_f">&nbsp;</td>' +
			           '<td id="'+this.stageId+'_1" class="droparea1"></td>' +
			           '<td id="'+this.stageId+'_2" class="droparea2"></td>' +
			           '<td id="'+this.stageId+'_3" class="droparea3"></td>' +
			           '</tr></table>';
			var div = document.createElement('div');
			div.setAttribute('id', this.stageId);
			div.innerHTML = html;
			cont.appendChild(div);

			$(this.stageId).addClassName('stagearea');
			this.createIframe(null,null, this.stageId+'_c1', this.stageId+'_1');
			this.createIframe(null,null, this.stageId+'_c2', this.stageId+'_2');
			this.createIframe(null,null, this.stageId+'_c3', this.stageId+'_3');
			this.stageLoaded = true;
		}
	},
	/**
	 * Restore all Objects in all stages, based on the informations we have in this.savedObjects
	 */
	restorePrintContent : function() {
		this.isPrintContent = true;
		this.restore();
	},

	/**
	 * Create an IFrame on which we can drop images and arange them after to a comic
	 */
	createIframe : function(w,h,id,parent, doreload) {
		var ifr = '<iframe src="blank.html" name="'+id+'" id="'+id+'" frameborder="0" allowtransparency="true" scrolling="no" class="stageifr" style="';
		if (w != null) ifr += 'width:' + w + 'px;';
		if (h != null) ifr += 'height:' + h + 'px;';
		ifr += '"></iframe>';
		$(parent).update(ifr);
		if (Prototype.Browser.KHTML || Prototype.Browser.WebKit) {
			window.setTimeout("$('"+id+"').src='blank.html';", 100);
		}
	},
	/**
	 * Return the Body-Element from the IFrame
	 * param string id The Stage-IFrame-ID from which we want to get the BodyElement
	 * return BodyElement The HTML-BodyElement from the IFrame
	 */
	getIframeBody : function(id) {
		return this.getIframeDoc(id).body;
	},
	/**
	 * Return the Document from the IFrame
	 * param string id The Stage-IFrame-ID from which we want to get the DocumentElement
	 * return DocumentElement The HTML-DocumentElement from the IFrame
	 */
	getIframeDoc : function(id) {
		return this.getIframeWin(id).document;
	},
	/**
	 * Reutn the WindowElement from the IFrame
	 * param string id The Stage-IFrame-ID from which we want to get the WindowElement
	 * return WindowElement The HTML-WindowElement from the IFrame
	 */
	getIframeWin : function(id) {
		return $(id).contentWindow;
	},
	/**
	 * Return true/false if all StageFrames are loaded
	 * return boolean true if all stages are loaded
	 */
	stageFramesLoaded : function() {
		var loaded = true;
		if (!$(this.stageId+'_c1') || !this.getIframeWin(this.stageId+'_c1') || !this.getIframeWin(this.stageId+'_c1').frameLoaded) {
			loaded = false;
		}
		if (loaded && (!$(this.stageId+'_c2') || !this.getIframeWin(this.stageId+'_c2') || !this.getIframeWin(this.stageId+'_c2').frameLoaded) ) {
			loaded = false;
		}
		if (loaded && (!$(this.stageId+'_c3') || !this.getIframeWin(this.stageId+'_c3') || !this.getIframeWin(this.stageId+'_c3').frameLoaded) ) {
			loaded = false;
		}
		return loaded;
	},

	/**
	 * Make a StageArea dropable for images
	 */
	makeDropable : function() {
		this.getDropables().each(function(id) {
			Droppables.add(id, {accept:'dragable', hoverclass:'stagehover',onDrop: function(elem, area) {
				ComicStage.appendToStageComponent(elem, area);
			}});
		});
	},
	/**
	 * Disable all Dropareas on this stage - mostly called before remove a stage
	 */
	removeDropables : function() {
		this.getDropables().each(function(id) {
			Droppables.remove($(id));
		});
	},
	/**
	 * Return an Object with all Stageareas on which we can drop an image
	 * return Object All areas on which we can drop an image
	 */
	getDropables : function() {
		return [ this.stageId+'_1', this.stageId+'_2', this.stageId+'_3' ];
	},

	/**
	 * Called after dropping an Image on a StageArea - calls a function inside the iframe to add the image on this stage
	 */
	appendImage : function(img, num) {
		var stage = this.stageId + '_c' + num;
		var posImg = Position.cumulativeOffset(img);
		var posStg = Position.cumulativeOffset($(this.stageId + '_' + num));
		var chooser = ComicStage.stageElements[img.getAttribute('longdesc')];

		// We need to add the Scroll-Positions to the images - this is different in IE so we need to check for it
		if (Prototype.Browser.IE) {
			posImg[0] += document.documentElement.scrollLeft;
			posImg[1] += document.documentElement.scrollTop;
		} else {
			posImg[0] += window.pageXOffset;
			posImg[1] += window.pageYOffset;
		}

		this.getIframeWin(stage).addImage(img, chooser, stage, posImg, posStg);
	},

	/**
	 * Get all Objects from each Stage and return an Object with all of them
	 * We use this for saving and for moving stages in position
	 * return Object An Object with an entry for each Stage
	 */
	getStageElements : function() {
		var ifr, back = {};
		for (var i = 1; i < 4; i++) {
			back['stage_' + i] = this.getIframeWin(this.stageId + '_c' + i).getObjects();
		}
		return back;
	},

	/**
	 * Just the last function ;-) so we can just add a commma after each function....
	 */
	last : null
};