/**
 *
 * Copyright (c) 2007 Zurschmiede Lukas
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 *
 *
 * ComicMaker:ComicMaker_StageImage
 * http://www.delight.ch/
 * Zurschmiede Lukas
 * delight software gmbh
 * Version 1.0 (05-07-2007)
 *
 **/
function ComicMaker_StageImage(name, src, stageId) {
	this.src = src;
	this.id = 'notset';
	this.posX = 0;
	this.posY = 0;
	this.width = 0;
	this.height = 0;
	this.zIndex = 100;
	this.selectBorderWidth = 2;
	this.stageId = stageId;
	this.name = name;
	this.imageTextEditable = false;
	this.imageResizable = true;
	this.imageText = '';
	this.drag = null;
	this.isResizing = false;
	this.resizeProcess = false;
	this.resizePositions = {x:0, y:0}
	this.resizeProportional = true;
	this.menu = new djs_ContextMenu();
	this.isPrintableContent = false;
	this.menu.init({
		resize :     {name:"resize",     text:"Change dimension", icon:"resize.png"},
		remove :     {name:"delete",     text:"Remove",           icon:"trash.png"},
		spacer1 :    {spacer:true},
		front :      {name:"front",      text:"Bring to front",   icon:"front.png"},
		oneup :      {name:"oneup",      text:"One layer up",     icon:"onefront.png"},
		onedown :    {name:"onedown",    text:"One layer down",   icon:"oneback.png"},
		background : {name:"background", text:"Bring to back",    icon:"back.png"}
	});
}

ComicMaker_StageImage.prototype = {
	setPosition : function(x,y) {
		this.posX = x;
		this.posY = y;
	},
	setDimension : function(w,h) {
		this.width = w;
		this.height = h;
	},
	setId : function(id) {
		this.id = id;
	},
	setZIndex : function(idx) {
		this.zIndex = idx;
		if ($(this.id)) {
			$(this.id).style.zIndex = idx;
		}
	},
	setEditable : function(edt) {
		this.imageTextEditable = edt;
	},
	setResizable : function(res) {
		this.imageResizable = res;
		if (!res) {
			this.menu.remove('resize');
		}
	},
	setImageText : function(txt) {
		this.imageText = txt;
	},
	setIsPrintableContent : function(printable) {
		this.isPrintableContent = printable;
	},

	getStageId : function() {
		return this.stageId;
	},
	getName : function() {
		return this.name;
	},
	getSource : function() {
		return this.src;
	},
	getId : function() {
		return this.id;
	},
	getZIndex : function() {
		return this.zIndex;
	},
	isEditable : function() {
		return this.imageTextEditable;
	},
	isResizable : function() {
		return this.imageResizable;
	},
	getImageText : function() {
		return this.imageText;
	},
	getDimension : function() {
		return {width:this.width, height:this.height};
	},

	showMenu : function(x, y) {
		this.callAction('select');
		this.menu.showMenu(x, y, document.getElementsByTagName('body')[0], this.id);
	},
	removeMenu : function(id) {
		this.callAction('unselect');
		this.menu.removeMenu(id);
	},

	showImage : function() {
		if (this.isEditable()) {
			var simg = document.createElement('div');
			simg.style.textAlign = 'center';
			simg.style.verticalAlign = 'middle';
			simg.innerHTML = this.getImageText();
			simg.style.background = 'url('+this.src+') top left no-repeat';
			simg.style.width = this.width + 'px';
			simg.style.height = this.height + 'px';
			simg.style.padding = '2px';
			simg.style.fontSize = '8pt';
		} else {
			var simg = document.createElement('img');
			simg.setAttribute('src', this.src);
		}
		simg.setAttribute('id',  this.id);
		simg.style.position = 'absolute';
		simg.style.display = 'block';
		simg.style.left = this.posX + 'px';
		simg.style.top = this.posY + 'px';
		simg.style.zIndex = this.zIndex;

		simg = document.getElementsByTagName('body')[0].appendChild(simg);

		Element.makePositioned(simg);
		if (!this.isPrintableContent) {
			this.drag = new Draggable(simg.id, {ghosting:false});
			//Event.observe(simg, 'mousedown', imageEvents.doShowMenu);
			Event.observe(simg, 'contextmenu', imageEvents.doShowMenu);
			if (this.isEditable()) {
				Event.observe(simg, 'dblclick', imageEvents.doShowTextEdit);
			}
		}
	},

	callAction : function(action) {
		switch (action.toLowerCase()) {
			case 'select':
				$(this.id).style.border = this.selectBorderWidth + 'px dotted rgb(150,150,150)';
				$(this.id).style.top = (parseInt($(this.id).style.top) - this.selectBorderWidth) + 'px';
				$(this.id).style.left = (parseInt($(this.id).style.left) - this.selectBorderWidth) + 'px';
				break;

			case 'unselect':
				$(this.id).style.borderWidth = '0px';
				$(this.id).style.top = (parseInt($(this.id).style.top) + this.selectBorderWidth) + 'px';
				$(this.id).style.left = (parseInt($(this.id).style.left) + this.selectBorderWidth) + 'px';
				break;

			case 'front':
				var obj = this;
				images.each(function(img, key) {
					if (img.getZIndex() >= obj.getZIndex()) {
						obj.setZIndex(img.getZIndex() + 1);
					}
				});
				break;

			case 'background':
				var obj = this;
				images.each(function(img, key) {
					if (img.getZIndex() <= obj.getZIndex()) {
						obj.setZIndex(img.getZIndex() - 1);
					}
				});
				break;

			case 'oneup':
				var imgs = {};
				var zIndex = this.getZIndex();
				images.each(function(img, key) {
					if (img.getZIndex() > zIndex) {
						imgs['img' + img.getZIndex()] = key;
					}
				});
				var keys = $H(imgs).keys().sort();
				if (keys.length > 0) {
					var newidx = images[imgs[keys[0]]].getZIndex();
					images[imgs[keys[0]]].setZIndex(this.getZIndex());
					this.setZIndex(newidx);
				}
				imgs = null;
				break;

			case 'onedown':
				var imgs = {};
				var zIndex = this.getZIndex();
				images.each(function(img, key) {
					if (img.getZIndex() < zIndex) {
						imgs['img' + img.getZIndex()] = key;
					}
				});
				var keys = $H(imgs).keys().sort();
				if (keys.length > 0) {
					var newidx = images[imgs[keys[keys.length - 1]]].getZIndex();
					images[imgs[keys[keys.length - 1]]].setZIndex(this.getZIndex());
					this.setZIndex(newidx);
				}
				imgs = null;
				break;

			case 'delete':
				this.removeMenu(contextMenuShow);
				var obj = this.getId();
				var newimages = new Array()
				images.each(function (img) {
					if (img.getId() != obj) {
						newimages.push(img);
					}
				});
				images = newimages;
				$(obj).parentNode.removeChild($(obj));
				break;

			case 'resize':
				this.isResizing = true;
				var obj = this.getId();
				var imgObj = 0; images.each(function(img, key) { if (img && (img.getId() == obj)) {imgObj = key; return true} });
				var dim = $(this.getId()).getDimensions();
				var imgstyle = 'position:absolute;display:block;top:' + (parseInt($(this.getId()).style.top) - 8) + 'px;left:' + (parseInt($(this.getId()).style.left) - 8) + 'px;';

				var html = '<div id="'+this.getId()+'" style="'+imgstyle+'"><table cellpadding="0" cellspacing="0"><tr>' +
				           '<td style="background:url(images/layout/arrow_topleft.png) top left no-repeat;width:8px;font-size:1px;" onmousedown="imageEvents.resizeProcess('+imgObj+',\'tl\');">&nbsp;</td>' +
				           '<td style="width:'+dim.width+'px;height:8px;font-size:1px;">&nbsp;</td>' +
				           '<td style="background:url(images/layout/arrow_topright.png) top left no-repeat;width:8px;font-size:1px;" onmousedown="imageEvents.resizeProcess('+imgObj+',\'tr\');">&nbsp;</td>' +
				           '</tr><tr>' +
				           '<td style="width:8px;font-size:1px;">&nbsp;</td>' +
				           '<td style="width:'+dim.width+'px;font-size:1px;"><img id="img'+this.getId()+'" src="'+this.getSource()+'" style="width:'+dim.width+'px;height:'+dim.height+'px;" /></td>' +
				           '<td style="width:8px;font-size:1px;">&nbsp;</td>' +
				           '</tr><tr>' +
				           '<td style="background:url(images/layout/arrow_bottomleft.png) top left no-repeat;width:8px;font-size:1px;" onmousedown="imageEvents.resizeProcess('+imgObj+',\'bl\');">&nbsp;</td>' +
				           '<td style="width:'+dim.width+'px;height:8px;font-size:1px;">&nbsp;</td>' +
				           '<td style="background:url(images/layout/arrow_bottomright.png) top left no-repeat;width:8px;font-size:1px;" onmousedown="imageEvents.resizeProcess('+imgObj+',\'br\');">&nbsp;</td>' +
				           '</tr></table></div>';

				Event.stopObserving($(this.id), 'mousedown', imageEvents.doShowMenu);
				this.drag.destroy();
				$(this.id).replace(html);
				window.setTimeout("Event.observe(window, 'click', imageEvents.doResizeImage)", 500);
				break;

			case 'removeresize':
				this.isResizing = false;
				Event.stopObserving(window, 'click', imageEvents.doResizeImage);
				var dim = $(this.id).getDimensions();
				var top = parseInt($(this.id).style.top);
				var left = parseInt($(this.id).style.left);
				$(this.id).parentNode.removeChild($(this.id));
				this.showImage();
				$(this.id).style.width = (dim.width - 16) + 'px';
				$(this.id).style.height = (dim.height - 16) + 'px';
				$(this.id).style.top = (top + 8) + 'px';
				$(this.id).style.left = (left + 8) + 'px';
				break;

			case 'startresizing':
				var dim = $(this.getId()).getDimensions();
				this.resizeProcess = $A(arguments)[1];
				this.resizePositions.w = dim.width;
				this.resizePositions.h = dim.height;

				switch (this.resizeProcess) {
					case 'tl':
						this.resizePositions.x = parseInt($(this.getId()).style.left);
						this.resizePositions.y = parseInt($(this.getId()).style.top);
						break;
					case 'tr':
						this.resizePositions.x = parseInt($(this.getId()).style.left) + this.resizePositions.w;
						this.resizePositions.y = parseInt($(this.getId()).style.top);
						break;
					case 'bl':
						this.resizePositions.x = parseInt($(this.getId()).style.left);
						this.resizePositions.y = parseInt($(this.getId()).style.top) + this.resizePositions.h;
						break;
					case 'br':
						this.resizePositions.x = parseInt($(this.getId()).style.left) + this.resizePositions.w;
						this.resizePositions.y = parseInt($(this.getId()).style.top) + this.resizePositions.h;
						break;
				}

				Event.observe(window, 'mousemove', imageEvents.doResizeChangeSize);
				Event.observe(window, 'mouseup',   imageEvents.doResizeEnd);
				break;

			case 'doresize':
				var args  = $A(arguments);
				var newX  = args[1];
				var newY  = args[2];
				var dTop  = parseInt($(this.id).style.top);
				var dLeft = parseInt($(this.id).style.left);
				var diffX = this.resizePositions.x - newX;
				var diffY = this.resizePositions.y - newY;

				switch (this.resizeProcess) {
					case 'tl':
						if (this.resizeProportional) {

						} else {

						}
						$(this.id).style.width  = (this.resizePositions.w + diffX) + 'px';
						$(this.id).style.height = (this.resizePositions.h + diffY) + 'px';

						$('img' + this.id).style.width  = (this.resizePositions.w - 16 + diffX) + 'px';
						$('img' + this.id).style.height = (this.resizePositions.h - 16 + diffY) + 'px';

						$(this.id).style.left = newX + 'px';
						$(this.id).style.top  = newY + 'px';
						break;
					case 'tr':
						$(this.id).style.width  = (this.resizePositions.w - diffX) + 'px';
						$(this.id).style.height = (this.resizePositions.h + diffY) + 'px';

						$('img' + this.id).style.width  = (this.resizePositions.w - 16 - diffX) + 'px';
						$('img' + this.id).style.height = (this.resizePositions.h - 16 + diffY) + 'px';

						$(this.id).style.top  = newY + 'px';
						// left must not be changed, we change the size on right
						break;
					case 'bl':
						$(this.id).style.width  = (this.resizePositions.w + diffX) + 'px';
						$(this.id).style.height = (this.resizePositions.h - diffY) + 'px';

						$('img' + this.id).style.width  = (this.resizePositions.w - 16 + diffX) + 'px';
						$('img' + this.id).style.height = (this.resizePositions.h - 16 - diffY) + 'px';

						$(this.id).style.left = newX + 'px';
						// top must not be changed, we change the size on bottom
						break;
					case 'br':
						$(this.id).style.width  = (this.resizePositions.w - diffX) + 'px';
						$(this.id).style.height = (this.resizePositions.h - diffY) + 'px';

						$('img' + this.id).style.width  = (this.resizePositions.w - 16 - diffX) + 'px';
						$('img' + this.id).style.height = (this.resizePositions.h - 16 - diffY) + 'px';
						// reposition not needed: top left corner will not move
						break;
				}

				break;

			case 'endresizing':
				this.resizeProcess = false;
				this.resizePositions.x = 0;
				this.resizePositions.y = 0;
				this.resizePositions.w = 0;
				this.resizePositions.h = 0;
				Event.stopObserving(window, 'mousemove', imageEvents.doResizeChangeSize);
				Event.stopObserving(window, 'mouseup',   imageEvents.doResizeEnd);

				var dim = Element.getDimensions($(this.getId()));
				this.setDimension(dim.width, dim.height);
				break;

			case 'showedit':
				Event.stopObserving($(this.id), 'dblclick', imageEvents.doShowTextEdit);
				var text = $(this.id).innerHTML;
				$(this.id).innerHTML = '<textarea name="edt' + this.id + '" id="edt' + this.id + '" style="width:'+(this.width-2)+'px;height:'+(this.height-2)+'px;border:1px solid red;background-color:transparent;font-size:8pt;text-align:center;">' + text.replace(/\<br\>/g,'\n') + '</textarea>';
				$('edt' + this.id).focus();
				$('edt' + this.id).select();
				Event.observe($('edt' + this.id), 'blur', imageEvents.doHideTextEdit);
				break;

			case 'hideedit':
				Event.stopObserving($('edt' + this.id), 'blur', imageEvents.doHideTextEdit);
				this.setImageText($('edt' + this.id).value.replace(/\n/g, '<br>'));
				$(this.id).innerHTML = this.getImageText();
				Event.observe($(this.id), 'dblclick', imageEvents.doShowTextEdit);
				break;
		}
	},

	getSaveObject : function() {
		var back = {background:false, image:'', name:this.name, id:0, top:0, left:0, width:0, height:0, pos:0, stage:this.stageId, imageText:'', imageTextEditable:false, imageResizable:true };
		var img = $(this.getId());
		var pos = Position.page(img);

		back.image = this.getSource();
		back.id = this.getId();
		back.top = pos[1];
		back.left = pos[0];
		back.width = this.width;
		back.height = this.height;
		back.pos = parseInt(img.style.zIndex);
		back.imageText = this.getImageText();
		back.imageTextEditable = this.isEditable();
		back.imageResizable = this.isResizable();

		return back;
	},


	last : null
};