/**
 *
 * Copyright (c) 2007 Zurschmiede Lukas
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 *
 *
 * djs_Properties
 * http://www.delight.ch/
 * Zurschmiede Lukas
 * delight software gmbh
 * Version 1.0 (05-07-2007)
 *
 **/
function djs_Properties() {
	this.options = [];
}

djs_Properties.prototype = {
	
	/**
	 * Set a new "value" to parameter "name"
	 * 
	 * param String name Name of Property to assign the value value
	 * param mixed value Value to set on Parameter name
	 * return void
	 */
	set : function(name,value) {
		if ((typeof(name) != 'undefined') && (typeof(value) != 'undefined')) {
			name = name.toLowerCase();
			this.options[name] = value;
		}
	},
	
	/**
	 * Return an option-value or the default od null
	 *
	 * param String name Name of the Options/Property to get
	 * param Mixed def [optional] DefaultValue to return if the Option was not found
	 * param String subname [optional] If this Property is an Object, with subname the Property subname from this Object will be returned (or default/null instead)
	 * return mixed the requested value, value from aram def or null instead
	 */
	get : function(name,def,subname) {
		if (typeof(name) == 'undefined') {
			return def;
		}
		name = name.toLowerCase();
		if (typeof(this.options[name]) != 'undefined') {
			var opt = this.options[name];
			if ((typeof(opt) == 'object') && (typeof(subname) != 'undefined')) {
				var val = typeof(def) != 'undefined' ? def : null;
				try {
					val = typeof(opt[subname]) != 'undefined' ? opt[subname] : val;
				} catch(e) {
					val = typeof(def) != 'undefined' ? def : null;
				}
				return val;
			} else {
				return opt;
			}
		} else {
			return typeof(def) != 'undefined' ? def : null;
		}
		return null;
	},
	
	/**
	 * Return all Keys from properties
	 *
	 * return Array Array with all keys
	 */
	getKeys : function() {
		return $H(this.options).keys();
	},
	
	/**
	 * Parse an Object as Key-Value-Pair into the options
	 * 
	 * param Object arr Object which should be added to options
	 * return void
	 */
	parse : function(arr) {
		opt = this; // we cannot access to "this" inside the function so we point "this" to "opt"
		$H(arr).each(function(obj) {
			opt.set(obj.key, obj.value);
		});
	},
	
	last : null
};
