/**
 *
 * Copyright (c) 2007 Zurschmiede Lukas
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 *
 *
 * DOMExtended
 * http://www.delight.ch/
 * Zurschmiede Lukas
 * delight software gmbh
 * Version 1.0 (05-07-2007)
 *
 **/

// Some usefull methodts which we want on Prototypes Element-Object
DOMExtended = {
	// Insert an Element after an other - insertBefore exists already in DOM, but not insertAfter
	insertAfter : function(element, node) {
		if ($(element).parentNode.lastChild == $(element)) {
			return $(element).parentNode.appendChild(node);
		} else {
			return $(element).parentNode.insertBefore(node, $(element).nextSibling);
		}
	},
	
	// Replace an Element with an other - Prototype has already replace, but there we can just replace HTML (innerHTML)
	replaceElement : function(element, node) {
		$(element).parentNode.replaceChild(node, element);
		return node;
	},
	
	// Change the parent from a DOMElement - Trough this function we can move content from one Element to an other
	moveNode : function(element, node) {
		return $(node).appendChild($(element));
	},
	
	// Return the next Element (Next Sibling, which isn't a TEXT-Node)
	nextElement : function(element) {
		element = $(element);
		while (element = element.nextSibling) {
			if (element.nodeType == 1) {
				return $(element);
			}
		}
		return null;
	},
	
	// Return the previous Element (NPrevious Sibling, which isn't a TEXT-Node)
	previousElement : function(element) {
		element = $(element);
		while (element = element.previousSibling) {
			if (element.nodeType == 1) {
				return $(element);
			}
		}
		return null;
	}
};

// Add them to Prototypes-Element
Element.addMethods(DOMExtended);
